
# Reciprocity and behavioral heterogeneity govern the stability of social networks 
# Roslyn Dakin and T. Brandt Ryder

# Can you set up a de novo simulation?

# Set up data frame of repeataed measures of individuals per recording sessions:

library(dplyr)
library(lme4)
library(lmerTest)
library(visreg)
library(igraph)

load('manakin_data_individual_tradeoff_analysis.RData')

# Set up the data that will be used for plotting:
ndat1 <- data.frame(expand.grid(mean_L.n.partnerships=seq(0.09, 2.7, by=0.1), rel_L.n.partnerships=0))
ndat2 <- data.frame(expand.grid(mean_L.n.partnerships=1.7, rel_L.n.partnerships=seq(-2.1,1.8,by=0.1)))
head(ndat1); str(ndat1); head(ndat2); str(ndat2)
ndat1$studyyear <- '16-17'; ndat1$Status <- 'terr'; ndat1$sample_eff <- 74
ndat2$studyyear <- '16-17'; ndat2$Status <- 'terr'; ndat2$sample_eff <- 74

mod.raw # Results of the empirical data model fit

# Get social interaction data:
year3 <- read.csv('filtered_interactions_nodups_times_durations_052418.csv', stringsAsFactors=F)[,c('date','nTime','Color1','Color2','Lek','studyyear','DL','sitefix')]; dim(year3)
year12 <- read.csv('filtered_interactions_nodups_times_durations_011018.csv', stringsAsFactors=F)[,c('date','nTime','Color1','Color2','Lek','studyyear','DL','sitefix')]; dim(year12)
ally <- rbind(year12, year3); head(ally); dim(ally) # Duplicate and reverse individual 1 & individual 2 to account for all interactions:
allyX <- ally
allyX$Color1 <- ally$Color2; allyX$Color2 <- ally$Color1
dim(ally); dim(allyX)
ally <- rbind(ally, allyX)
ally$Color1[ally$Color1=='OX-'] <- 'OX' # Correct ID error
ally$Color2[ally$Color2=='OX-'] <- 'OX'
ally$sitefix[ally$sitefix=='OX-'] <- 'OX'
all(levels(factor(ally$Color1))==levels(factor(ally$Color2))); dim(ally) # 73,770

head(bird.dat); dim(bird.dat)
bird.dat.sim <- bird.dat[,c(1:3,6,7)]
head(bird.dat.sim)
bird.dat.sim$n.partnerships <- NA; bird.dat.sim$prop.persisting <- NA; head(bird.dat.sim)

nperm <- 1000
perm.results.sim <- data.frame(among.coef=rep(NA, nperm), within.coef=NA, intercept=NA, among.p=NA, within.p=NA)
dim(perm.results.sim); head(perm.results.sim)

perm.fit.among.0 <- vector('list', length=nperm); perm.fit.among.1 <- vector('list', length=nperm); perm.fit.among.2 <- vector('list', length=nperm)
perm.fit.within.0 <- vector('list', length=nperm); perm.fit.within.1 <- vector('list', length=nperm); perm.fit.within.2 <- vector('list', length=nperm)

dls <- unique(ally[,c('Lek','studyyear','DL')]); head(dls); dim(dls)
rownames(dls) <- 1:nrow(dls)


for(j in 1:nperm){
	
	set.seed(j)
	newg.list <- vector('list', length=nrow(dls))
	for(i in 1:nrow(dls)){
		
		lk <- dls$Lek[i]
		dl <- dls$DL[i]
		yr <- dls$studyyear[i]
		g1.0 <- subset(ally, studyyear==yr & Lek==lk & DL==dl)
		
		g1a <- unique(g1.0[,c('Color1','Color2')])
		g1a <- subset(g1a, Color1<Color2) 
		temp <- summarize(group_by(g1.0, Color1, Color2), n=n())
		g1a$raw.weight <- temp$n[match(paste(g1a$Color1, g1a$Color2),  paste(temp$Color1, temp$Color2))]
		g1w <- graph.data.frame(data.frame(Color1=g1a$Color1, Color2=g1a$Color2, weight=g1a$raw.weight), directed=F) # Net
		g2w <- rewire(g1w, each_edge(prob=1, loops=F))

		g2temp <- data.frame(as_edgelist(g2w), stringsAsFactors=F)	
		g2adj <- g2temp; g2adj$X1 <- g2temp$X2; g2adj$X2 <- g2temp$X1
		newg <- rbind(g2temp, g2adj)
		names(newg) <- c('Color1','Color2')
		newg$Lek <- lk; newg$DL <- dl; newg$studyyear <- yr
		newg.list[[i]] <- newg
	}
	newg.list <- data.table::rbindlist(newg.list)
	
	for(i in 1:nrow(bird.dat.sim)){ #
		ID <- bird.dat.sim$Color1[i]
		dl <- bird.dat.sim$DL[i]
		temp1 <- subset(newg.list, Color1==ID & DL==dl)
		temp2 <- subset(newg.list, Color1==ID & DL==dl+1)

		bird.dat.sim$n.partnerships[i] <- length(table(temp1$Color2))
		# Persistence of partnerships:	
		if(nrow(subset(newg.list, Lek==lk & DL==dl))>0){
			bird.dat.sim$prop.persisting[i] <- sum(names(table(temp1$Color2)) %in% names(table(temp2$Color2))) / length(table(temp1$Color2)) 		
		}
	}

	# Within-individual centering to get a relative measure of degree (number of partnerships): 
	bird.avg.sim <- summarize(group_by(bird.dat.sim, Color1), mean_L.n.partnerships=mean(log(n.partnerships+0.1)))
	bird.datX.sim <- merge(bird.dat.sim, bird.avg.sim)
	bird.datX.sim$rel_L.n.partnerships <- log(bird.datX.sim$n.partnerships+0.1) - bird.datX.sim$mean_L.n.partnerships # Relative degree

	# Fit the model:
	summary(modX <- glmer(cbind(round(prop.persisting*n.partnerships), round((1 - prop.persisting)*n.partnerships)) ~ studyyear + Status + sample_eff + (mean_L.n.partnerships) + (rel_L.n.partnerships) + (1|Color1), data=subset(bird.datX.sim, !is.na(prop.persisting)), family='binomial'))

	perm.results.sim$among.coef[j] <- fixef(modX)['mean_L.n.partnerships']
	perm.results.sim$within.coef[j] <- fixef(modX)['rel_L.n.partnerships']
	perm.results.sim$intercept[j] <- summary(modX)$coef[1,1]
	perm.results.sim$among.p[j] <- summary(modX)$coef[6,'Pr(>|z|)']
	perm.results.sim$within.p[j] <- summary(modX)$coef[7,'Pr(>|z|)']
	
	perm.fit.among.0[[j]] <- predict(modX, newdata=ndat1, re.form=NA, type='response')	
	perm.fit.within.0[[j]] <- predict(modX, newdata=ndat2, re.form=NA, type='response')

	perm.fit.among.1[[j]] <- predict(modX, newdata=ndat1.1, re.form=NA, type='response')	
	perm.fit.among.2[[j]] <- predict(modX, newdata=ndat1.2, re.form=NA, type='response')	

	perm.fit.within.1[[j]] <- predict(modX, newdata=ndat2.1, re.form=NA, type='response')
	perm.fit.within.2[[j]] <- predict(modX, newdata=ndat2.2, re.form=NA, type='response')

	print(j)
}


# Get an average fit & confidence interval for the permutation analysis:
p.f.a <- (as.data.frame(perm.fit.among.0))
p.f.w <- (as.data.frame(perm.fit.within.0))
rowMeans(p.f.a)
rowMeans(p.f.w)
p.f.a.1 <- (as.data.frame(perm.fit.among.1))
p.f.a.2 <- (as.data.frame(perm.fit.among.2))
p.f.w.1 <- (as.data.frame(perm.fit.within.1))
p.f.w.2 <- (as.data.frame(perm.fit.within.2))



par(mfrow=c(2,2), bty='l', las=1, mar=c(4,4,0.1,0.1))
plot(predict(mod.raw, newdata=ndat1, re.form=NA, type='response') ~ ndat1$mean_L.n.partnerships, pch=16, type='l', ylim=c(0,1), ylab='Predicted proportion maintained', xlab='Average degree')
ndat1.1 <- ndat1
ndat1.1$rel_L.n.partnerships <- -2.1
ndat1.2 <- ndat1
ndat1.2$rel_L.n.partnerships <- 1.8
polygon(x=c(ndat1.1$mean_L.n.partnerships, rev(ndat1.1$mean_L.n.partnerships)), y=c(predict(mod.raw, newdata=ndat1.1, re.form=NA, type='response'), rev(predict(mod.raw, newdata=ndat1.2, re.form=NA, type='response'))), border=NA, col=rgb(0,0,0,0.1))
points(rowMeans(p.f.a) ~ ndat1$mean_L.n.partnerships, type='l')
polygon(x=c(ndat1$mean_L.n.partnerships, rev(ndat1$mean_L.n.partnerships)), y=c(rowMeans(p.f.a.1), rev(rowMeans(p.f.a.2))), col=rgb(1,0,0,0.1), border=NA)
# polygon(x=c(ndat1$mean_L.n.partnerships, rev(ndat1$mean_L.n.partnerships)), y=c(apply(p.f.a.0, 1, 'quantile', 0.01), rev(apply(p.f.a.0, 1, 'quantile', 0.99))), col=rgb(1,0,0,0.1), border=NA)

legend('topleft', fill=c(rgb(0,0,0,0.1), rgb(1,0,0,0.1)), legend=c('Observed networks', 'Random networks'), bty='n', cex=0.75, border=NA)

# for(i in 1:nrow(subset(perm.results.sim, !is.na(among.coef)))){
	# points(perm.fit.among[[i]] ~ ndat1$mean_L.n.partnerships, type='l', col=rgb(1,0,0,0.15), lwd=0.2)
# }


plot(predict(mod.raw, newdata=ndat2, re.form=NA, type='response') ~ ndat2$rel_L.n.partnerships, pch=16, type='l', ylim=c(0,1), ylab='Predicted proportion maintained', xlab='Relative degree')
ndat2.1 <- ndat2
ndat2.1$mean_L.n.partnerships <- 0.09
ndat2.2 <- ndat2
ndat2.2$mean_L.n.partnerships <- 2.7
polygon(x=c(ndat2.1$rel_L.n.partnerships, rev(ndat2.1$rel_L.n.partnerships)), y=c(predict(mod.raw, newdata=ndat2.1, re.form=NA, type='response'), rev(predict(mod.raw, newdata=ndat2.2, re.form=NA, type='response'))), border=NA, col=rgb(0,0,0,0.1))
points(rowMeans(p.f.w) ~ ndat2$rel_L.n.partnerships, type='l')
polygon(x=c(ndat2$rel_L.n.partnerships, rev(ndat2$rel_L.n.partnerships)), y=c(rowMeans(p.f.w.1), rev(rowMeans(p.f.w.2))), col=rgb(1,0,0,0.1), border=NA)
# polygon(x=c(ndat2$rel_L.n.partnerships, rev(ndat2$rel_L.n.partnerships)), y=c(apply(p.f.w, 1, 'quantile', 0.01), rev(apply(p.f.w, 1, 'quantile', 0.99))), col=rgb(1,0,0,0.1), border=NA)

# for(i in 1:nrow(subset(perm.results.sim, !is.na(among.coef)))){
	# points(perm.fit.within[[i]] ~ ndat2$rel_L.n.partnerships, type='l', col=rgb(1,0,0,0.15), lwd=0.2)
# }


# save(perm.fit.among.0, perm.fit.within.0, perm.fit.among.1, perm.fit.within.1, perm.fit.among.2, perm.fit.within.2, perm.results.sim, file='manakin_data_individual_tradeoff_rewire_real_data.RData')








